@@ -11,6 +11,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <limits.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -1377,10 +1378,9 @@ void refresh(xcWidget bw, caddr_t clientdata, caddr_t calldata)
 
 void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   if (eventmode == NORMAL_MODE || eventmode == COPY2_MODE ||
-	 eventmode == PUSH_MODE || eventmode == PRESS_MODE ||
-	 eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
-	 eventmode == FONTCAT2_MODE) {
+   if (eventmode == NORMAL_MODE || eventmode == COPY_MODE ||
+	 eventmode == MOVE_MODE || eventmode == CATALOG_MODE ||
+	eventmode == FONTCAT_MODE || eventmode == EFONTCAT_MODE) {
 
       centerview(areastruct.topinstance);
       invalidate_graphics(topobject);
@@ -1694,6 +1694,72 @@ void UDrawBox(XPoint origin, XPoint corner)
 		worig.x, worig.y);
 }
 
+/*----------------------------------------------------------------------*/
+/* Draw a box indicating the dimensions of the edit element that most	*/
+/* closely reach the position "corner".					*/
+/*----------------------------------------------------------------------*/
+
+void UDrawRescaleBox(XPoint *corner)
+{
+   XPoint 	origpoints[5], newpoints[5];
+   genericptr	rgen;
+   float	savescale, newscale, lastscale, tempscale;
+   long		mindist, testdist, lastmindist;
+   labelptr	rlab;
+   int		i, j, idx;
+
+   /* To be done---draw a box around each selected part, not	*/
+   /* just the first one.					*/
+
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   
+   for (j = 0; j < areastruct.selects; j++) {
+      rgen = SELTOGENERIC(areastruct.selectlist + j);
+
+      switch(ELEMENTTYPE(rgen)) {
+         case LABEL:
+	    rlab = (labelptr)rgen;
+	    savescale = rlab->scale;
+	    newscale = lastscale = rlab->scale * 2;
+	    mindist = LONG_MAX;
+	    while (mindist > 4) {
+	       rlab->scale = newscale;
+	       labelbbox(rlab, newpoints, areastruct.topinstance);
+	       newpoints[5] = newpoints[0];
+	       lastmindist = mindist;
+	       mindist = LONG_MAX;
+	       for (i = 0; i < 4; i++) {
+	          testdist = finddist(&newpoints[i], &newpoints[i+1], corner);
+	          if (testdist < mindist)
+	             mindist = testdist;
+	       }
+	       if (mindist == lastmindist)
+		  break;
+	       else if (lastmindist == LONG_MAX)
+		  newscale /= 2;
+	       else {
+	          tempscale = (fabs)((newscale * lastmindist) - (lastscale * mindist)) / 
+				(abs)(lastmindist - mindist);
+	          lastscale = newscale;
+	          newscale = tempscale;
+	       }
+	    }
+	    rlab->scale = savescale;
+            break;
+         case GRAPHIC:
+            break;
+         case OBJINST:
+            break;
+         /* Note:  should be able to handle other elements, as well */
+      }
+
+      UTransformbyCTM(DCTM, newpoints, origpoints, 4);
+      strokepath(origpoints, 4, 0, 1);
+   }
+}
+
 /*-------------------------------------------------------------------------*/
 void UDrawBBox()
 {
