@@ -44,9 +44,12 @@ keybinding *keylist = NULL;
 
 extern Display *dpy;
 extern char  _STR[150], _STR2[250];
+extern short eventmode;
+extern Boolean pressmode;
 
 /*--------------------------------------------------------------*/
 /* Key modifiers (convenience definitions)			*/
+/* Use Mod2Mask for a "hold" ("press") definition.		*/
 /*--------------------------------------------------------------*/
 
 #define ALT (Mod1Mask << 16)
@@ -58,6 +61,7 @@ extern char  _STR[150], _STR2[250];
 #define BUTTON3 (Button3Mask << 16)
 #define BUTTON4 (Button4Mask << 16)
 #define BUTTON5 (Button5Mask << 16)
+#define HOLD (Mod2Mask << 16)
 
 /*--------------------------------------------------------------*/
 /* This list declares all the functions which can be bound to   */
@@ -79,15 +83,16 @@ static char *function_names[NUM_FUNCTIONS + 1] = {
    "Library Duplicate", "Library Hide", "Library Virtual Copy",
    "Page Directory", "Library Pop", "Virtual Copy",
    "Help", "Redraw", "View", "Zoom In", "Zoom Out", "Pan",
-   "Double Snap", "Halve Snap", "Pan Left", "Pan Right", "Pan Up",
-   "Pan Down", "Write", "Rotate", "Flip X", "Flip Y", "Snap", "Snap To",
+   "Double Snap", "Halve Snap", "Write", "Rotate", "Flip X",
+   "Flip Y", "Snap", "Snap To",
    "Pop", "Push", "Delete", "Select", "Box", "Arc", "Text",
    "Exchange", "Copy", "Move", "Join", "Unjoin", "Spline", "Edit",
    "Undo", "Redo", "Select Save", "Unselect", "Dashed", "Dotted",
-   "Solid", "Prompt", "Dot", "Wire", "Start", "Finish", "Cancel",
-   "Nothing", "Exit", "Netlist", "Swap", "Pin Label", "Pin Global",
-   "Info Label", "Graphic", "Rescale", "Connectivity", "Sim", "SPICE",
-   "PCB", "SPICE Flat", "Text Delete Param",
+   "Solid", "Prompt", "Dot", "Wire", "Cancel", "Nothing", "Exit",
+   "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label", "Graphic",
+   "Rescale", "Select Box", "Connectivity", "Continue Copy",
+   "Continue Element", "Finish Copy", "Finish Element", "Finish",
+   "Cancel Segment", "Sim", "SPICE", "PCB", "SPICE Flat", "Text Delete Param",
    NULL			/* sentinel */
 };
 
@@ -122,6 +127,25 @@ int firstbinding(int function)
    return -1;
 }
 
+/*--------------------------------------------------------------*/
+/* Return the nth function bound to the indicated keywstate.	*/
+/* Return -1 if no function is bound.				*/
+/*--------------------------------------------------------------*/
+
+int boundfunction(int keywstate, int n)
+{
+   int j;
+   keybinding *ksearch;
+
+   j = 0;
+   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
+      if (keywstate == ksearch->keywstate)
+	 if (j++ == n)
+	    return ksearch->function;
+
+   return -1;
+}
+
 /*--------------------------------------------------------------*/
 /* Return TRUE if the indicated key (keysym + bit-shifted state)*/
 /* is bound to the indicated function (called by name)		*/
@@ -234,6 +258,10 @@ int string_to_key(const char *keystring)
 	 keywstate |= ALT;
 	 kptr += 5;
       }
+      else if (!strncmp(kptr, "Hold_", 5)) {
+	 keywstate |= HOLD;
+	 kptr += 5;
+      }
       else if (*kptr == '^') {
 	 kptr++;
 	 ct = (int)tolower(*kptr);
@@ -300,11 +328,13 @@ char *binding_to_string(int keywstate, int function)
 	    tmpstr = function_names[ksearch->function];
 	 else
 	    tmpstr = key_to_string(ksearch->keywstate);
-	 retstr = (char *)realloc(retstr, strlen(retstr) + strlen(tmpstr) + 
+	 if (tmpstr != NULL) {
+	    retstr = (char *)realloc(retstr, strlen(retstr) + strlen(tmpstr) + 
 		((first) ? 1 : 3));
-	 if (!first) strcat(retstr, ", ");
-	 strcat(retstr, tmpstr);
-	 if (function >= 0) free(tmpstr);
+	    if (!first) strcat(retstr, ", ");
+	    strcat(retstr, tmpstr);
+	    if (function >= 0) free(tmpstr);
+	 }
 	 first = False;
       }
    }
@@ -331,12 +361,14 @@ char *key_to_string(int keywstate)
    kptr = (char *)malloc(32);
    kptr[0] = '\0';
    if (kmod & Mod1Mask) strcat(kptr, "Alt_");
+   if (kmod & Mod2Mask) strcat(kptr, "Hold_");
    if (kmod & ControlMask) strcat(kptr, "Control_");
    if (kmod & LockMask) strcat(kptr, "Capslock_");
    if (kmod & ShiftMask) strcat(kptr, "Shift_");
 
    if (str != NULL) {
-      kptr = (char *)realloc(kptr, strlen(str) + 32);
+      /* 33 is length of all modifiers concatenated + 1 */
+      kptr = (char *)realloc(kptr, strlen(str) + 33);
       strcat(kptr, str);
    }
    else {
@@ -534,8 +566,8 @@ void default_keybindings()
    add_vbinding(XK_KP_Prior, XCF_Justify, 8);
 
    add_binding(XK_Delete, XCF_Text_Delete);
-   add_binding(SHIFT | XK_Delete, XCF_Text_Delete_Param);
    add_binding(XK_Return, XCF_Text_Return);
+   add_binding(BUTTON1, XCF_Text_Return);
    add_binding(XK_BackSpace, XCF_Text_Delete);
    add_binding(XK_Left, XCF_Text_Left);
    add_binding(XK_Right, XCF_Text_Right);
@@ -573,6 +605,7 @@ void default_keybindings()
    add_binding(XK_i, XCF_Edit_Insert);
    add_binding(XK_Insert, XCF_Edit_Insert);
    add_binding(XK_e, XCF_Edit_Next);
+   add_binding(BUTTON1, XCF_Edit_Next);
    add_binding(XK_A, XCF_Attach);
    add_binding(XK_V, XCF_Virtual);
    add_binding(XK_l, XCF_Next_Library);
@@ -586,6 +619,7 @@ void default_keybindings()
    add_binding(XK_M, XCF_Library_Move);
    add_binding(XK_P, XCF_Page_Directory);
    add_binding(XK_less, XCF_Library_Pop);
+   add_binding(HOLD | BUTTON1, XCF_Library_Pop);
    add_binding(XK_h, XCF_Help);
    add_binding(XK_question, XCF_Help);
    add_binding(XK_space, XCF_Redraw);
@@ -595,13 +629,13 @@ void default_keybindings()
    add_binding(XK_v, XCF_View);
    add_binding(XK_Z, XCF_Zoom_In);
    add_binding(XK_z, XCF_Zoom_Out);
-   add_binding(XK_p, XCF_Pan);
+   add_vbinding(XK_p, XCF_Pan, 0);
    add_binding(XK_plus, XCF_Double_Snap);
    add_binding(XK_minus, XCF_Halve_Snap);
-   add_binding(XK_Left, XCF_Pan_Left);
-   add_binding(XK_Right, XCF_Pan_Right);
-   add_binding(XK_Up, XCF_Pan_Up);
-   add_binding(XK_Down, XCF_Pan_Down);
+   add_vbinding(XK_Left, XCF_Pan, 1);
+   add_vbinding(XK_Right, XCF_Pan, 2);
+   add_vbinding(XK_Up, XCF_Pan, 3);
+   add_vbinding(XK_Down, XCF_Pan, 4);
    add_binding(XK_W, XCF_Write);
    add_vbinding(XK_O, XCF_Rotate, -5);
    add_vbinding(XK_o, XCF_Rotate, 5);
@@ -615,7 +649,6 @@ void default_keybindings()
    add_binding(XK_Delete, XCF_Delete);
    add_binding(XK_d, XCF_Delete);
    add_binding(XK_F19, XCF_Select);
-   add_binding(BUTTON2, XCF_Select);
    add_binding(XK_b, XCF_Box);
    add_binding(XK_a, XCF_Arc);
    add_binding(XK_t, XCF_Text);
@@ -637,12 +670,13 @@ void default_keybindings()
    add_binding(XK_period, XCF_Dot);
    add_binding(BUTTON1, XCF_Wire);
    add_binding(XK_w, XCF_Wire);
-   add_binding(BUTTON3, XCF_Nothing);
    add_binding(CTRL | ALT | XK_q, XCF_Exit);
-   add_binding(BUTTON1, XCF_Start);
-   add_binding(SHIFT | BUTTON1, XCF_Finish);
-   add_binding(BUTTON2, XCF_Finish);
-   add_binding(BUTTON3, XCF_Cancel);
+   add_binding(HOLD | BUTTON1, XCF_Move);
+   add_binding(BUTTON1, XCF_Continue_Element);
+   add_binding(BUTTON1, XCF_Continue_Copy);
+   add_binding(BUTTON1, XCF_Finish);
+   add_binding(XK_Escape, XCF_Cancel);
+   add_binding(ALT | XK_r, XCF_Rescale);
    add_binding(ALT | XK_s, XCF_SnapTo);
    add_binding(ALT | XK_q, XCF_Netlist);
    add_binding(XK_slash, XCF_Swap);
@@ -659,8 +693,74 @@ void default_keybindings()
 /* add_binding(ALT | XK_f, XCF_SPICEflat);	*/
 /* add_binding(ALT | XK_p, XCF_PCB);		*/
 
+   /* 2-button vs. 3-button mouse bindings (set with -2	*/
+   /* commandline option; 3-button bindings default)	*/
+
+   if (pressmode) {
+      add_binding(BUTTON3, XCF_Text_Return);
+      add_binding(BUTTON3, XCF_Select);
+      add_binding(HOLD | BUTTON3, XCF_SelectBox);
+      add_binding(BUTTON3, XCF_Finish_Element);
+      add_binding(BUTTON3, XCF_Finish_Copy);
+
+      add_binding(XK_BackSpace, XCF_Cancel_Last);
+      add_binding(XK_BackSpace, XCF_Cancel);
+   }
+   else {
+      add_binding(BUTTON2, XCF_Text_Return);
+      add_binding(BUTTON2, XCF_Select);
+      add_binding(HOLD | BUTTON2, XCF_SelectBox);
+      add_binding(BUTTON2, XCF_Finish_Element);
+      add_binding(BUTTON2, XCF_Finish_Copy);
+      add_binding(BUTTON3, XCF_Cancel_Last);
+      add_binding(BUTTON3, XCF_Cancel);
+   }
 }
 
+#ifndef TCL_WRAPPER
+/*----------------------------------------------*/
+/* Mode-setting rebindings (non-Tcl version)	*/
+/*----------------------------------------------*/
+
+static int button1mode = XCF_Wire;
+
+/*--------------------------------------------------------------*/
+/* Re-bind BUTTON1 to the indicated function and optional value */
+/*--------------------------------------------------------------*/
+
+void mode_rebinding(int newmode, int newvalue)
+{
+   remove_binding(BUTTON1, button1mode);
+   add_vbinding(BUTTON1, newmode, newvalue);
+   button1mode = newmode;
+   toolcursor(newmode);
+}
+
+/*--------------------------------------------------------------*/
+/* Execute the function associated with the indicated BUTTON1	*/
+/* mode, but return the keybinding to its previous state.	*/
+/*--------------------------------------------------------------*/
+
+void mode_tempbinding(int newmode, int newvalue)
+{
+   int saveval;
+   XPoint cpos;
+
+   if (isnbound(BUTTON1, button1mode, &saveval)) {
+      remove_binding(BUTTON1, button1mode);
+      add_vbinding(BUTTON1, newmode, newvalue);
+      cpos = UGetCursor();
+      eventdispatch(BUTTON1, (int)cpos.x, (int)cpos.y);
+      remove_binding(BUTTON1, newmode);
+      add_vbinding(BUTTON1, button1mode, saveval);
+   }
+   else
+      fprintf(stderr, "Error: No such button1 binding %s\n",
+		func_to_string(button1mode));
+}
+
+#endif /* TCL_WRAPPER */
+
 #undef ALT
 #undef CTRL
 #undef CAPSLOCK
