@@ -231,10 +231,6 @@ void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (areastruct.selects > 0)
       connectivity(button, clientdata, calldata);
-   else if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 on wire to see connectivity.");
-      eventmode = CONNECT_MODE;
-   }
 }
 
 /*----------------------------------------------------------------------*/
@@ -828,7 +824,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 
    if (areastruct.schemon == False) return;
 
-   if (eventmode == PRESS_MODE || eventmode == COPY2_MODE)
+   if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
       standard_element_delete(NORMAL);
 
    /* Create symbol or schematic, if allowed by mode */
@@ -1073,12 +1069,12 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
       if (topobject->schemtype == PRIMARY) {
 	 /* Find a symbol to associate */
 	 startcatalog(w, LIBLIB, NULL);
-	 Wprintf("Click on library page, then symbol to associate.");
+	 Wprintf("Select library page, then symbol to associate.");
       }
       else {
 	 /* Find a schematic (page) to associate */
 	 startcatalog(w, PAGELIB, NULL);
-	 Wprintf("Click on schematic page to associate.");
+	 Wprintf("Select schematic page to associate.");
       }
    }
 }
