@@ -129,15 +129,12 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 /* Names used for convenience throughout the source code */
 
 #define topobject	areastruct.topinstance->thisobject
+#define hierobject	areastruct.hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   ((areastruct.drawstack != NULL) ? \
-			OBJINST : \
-			((*(topobject->plist + *(a)))->type & ALL_TYPES))
-#define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
-#define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
+
 #define ENDPART		topobject->plist + topobject->parts
 #define EDITPART	topobject->plist + areastruct.editpart
 
@@ -164,21 +161,26 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define TOSPLINE(a)	(*((splineptr *)(a))) 
 #define TOPATH(a)	(*((pathptr *)(a)))
 #define TOGRAPHIC(a)	(*((graphicptr *)(a)))
+#define TOGENERIC(a)	(*((genericptr *)(a)))
 
 /* conversions from a selection to a specific type */
 
-#define SELTOPOLY(a)	TOPOLY(topobject->plist + (*(a)))
-#define SELTOLABEL(a)	TOLABEL(topobject->plist + (*(a)))
-#define SELTOOBJINST(a)	((areastruct.drawstack != NULL) ? \
-			areastruct.drawstack->thisinst : \
-			TOOBJINST(topobject->plist + (*(a))))
-#define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
-#define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
-#define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
-#define SELTOGRAPHIC(a)	TOGRAPHIC(topobject->plist + (*(a)))
-#define SELTOGENERIC(a) ((areastruct.drawstack != NULL) ? \
-			(genericptr)areastruct.drawstack->thisinst : \
-			(*(topobject->plist + (*(a)))))
+#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+			(topobject->plist + *(a)) : \
+			(hierobject->plist + *(a)))
+
+#define SELTOPOLY(a)	TOPOLY(SELTOGENERICPTR(a))
+#define SELTOLABEL(a)	TOLABEL(SELTOGENERICPTR(a))
+#define SELTOOBJINST(a)	TOOBJINST(SELTOGENERICPTR(a))
+#define SELTOARC(a)	TOARC(SELTOGENERICPTR(a))
+#define SELTOSPLINE(a)	TOSPLINE(SELTOGENERICPTR(a))
+#define SELTOPATH(a)	TOPATH(SELTOGENERICPTR(a))
+#define SELTOGRAPHIC(a)	TOGRAPHIC(SELTOGENERICPTR(a))
+#define SELTOGENERIC(a)	TOGENERIC(SELTOGENERICPTR(a))
+
+#define SELECTTYPE(a)   ((SELTOGENERIC(a))->type & ALL_TYPES)
+#define SELTOCOLOR(a)	((SELTOGENERIC(a))->color)
+
 
 /* creation of new elements */
 
@@ -1309,7 +1311,7 @@ typedef struct {
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
-   pushlistptr	drawstack;
+   pushlistptr	hierstack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
